/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.viewer;

import it.tidalwave.metadata.NoSuchMetadataTypeException;
import java.io.IOException;
import javax.swing.JComponent;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import it.tidalwave.metadata.Metadata;
import it.tidalwave.metadata.MetadataItemHolder;
import it.tidalwave.metadata.spi.TestLoggingSupport;
import static it.tidalwave.metadata.Metadata.StorageType.*;
import it.tidalwave.metadata.viewer.mock.PaneMock;
import it.tidalwave.metadata.viewer.mock.MetadataPanelProviderMock;
import it.tidalwave.metadata.viewer.mock.DataObjectMock;
import it.tidalwave.metadata.viewer.mock.MetadataItemMock;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
        
/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataPanelProviderSupportTest extends TestLoggingSupport
  {
    private MetadataPanelProviderSupport panelProvider;
    private DataObject dataObjectWithNoMetadata;
    private DataObject dataObjectWithItem;
    private DataObject dataObjectWithNoItem;
    
    @Before
    public void setupFixture() 
      throws IOException, NoSuchMetadataTypeException 
      {
        panelProvider = new MetadataPanelProviderMock();
        FileUtil.setMIMEType("test", "image/test");
        final FileSystem fileSystem = FileUtil.createMemoryFileSystem();
        final FileObject root = fileSystem.getRoot();
        
        dataObjectWithNoMetadata = DataObject.find(root.createData("nometadata.test"));
        assertTrue(dataObjectWithNoMetadata instanceof DataObjectMock);
        assertNull(dataObjectWithNoMetadata.getLookup().lookup(Metadata.class));

        dataObjectWithItem = DataObject.find(root.createData("item.test"));
        assertTrue(dataObjectWithItem instanceof DataObjectMock);
        Metadata metadata = dataObjectWithItem.getLookup().lookup(Metadata.class);
        assertNotNull(metadata);
        MetadataItemHolder<MetadataItemMock> holder = metadata.findOrCreateItem(MetadataItemMock.class, INTERNAL);
        assertNotNull(holder);
        assertTrue(holder.isAvailable());

        dataObjectWithNoItem = DataObject.find(root.createData("noitem.test"));
        assertTrue(dataObjectWithNoItem instanceof DataObjectMock);
        metadata = dataObjectWithNoItem.getLookup().lookup(Metadata.class);
        assertNotNull(metadata);
        holder = metadata.findOrCreateItem(MetadataItemMock.class, INTERNAL);
        assertNotNull(holder);
        assertFalse(holder.isAvailable());
      }

    @After
    public void tearDown() 
      {
        panelProvider = null;
      }

    @Test
    public void testGetDisplayName() 
      {
        assertEquals("mock", panelProvider.getDisplayName());
      }

    @Test
    public void testIsDataObjectSupported() 
      {
        assertTrue(panelProvider.isDataObjectSupported(dataObjectWithItem)); 
        assertTrue(panelProvider.isDataObjectSupported(dataObjectWithNoItem)); 
        assertFalse(panelProvider.isDataObjectSupported(dataObjectWithNoMetadata)); 
      }

    @Test
    public void testCreatePanel() 
      {
        final JComponent component1 = panelProvider.createPanel();
        assertNotNull(component1);
        final JComponent component2 = panelProvider.createPanel();
        assertNotNull(component2);
        assertTrue(component1 == component2);
      }

    @Test
    public void setDataObjectWithItem() 
      throws Exception
      {
        final PaneMock pane = (PaneMock)panelProvider.createPanel();
        assertNotNull(pane);
        panelProvider.setDataObject(dataObjectWithItem);
        assertNotNull(pane.item);
        assertEquals("the name", pane.item.getName()); // it's the item created by the source
      }
    
    @Test
    public void setDataObjectWithNoMetadata() 
      throws Exception
      {
        final PaneMock pane = (PaneMock)panelProvider.createPanel();
        assertNotNull(pane);
        panelProvider.setDataObject(dataObjectWithNoMetadata);
        assertFalse(pane.item.isAvailable()); // means the item has been newly created
      }
    
    @Test
    public void setDataObjectWithNoItem() 
      throws Exception
      {
        final PaneMock pane = (PaneMock)panelProvider.createPanel();
        assertNotNull(pane);
        panelProvider.setDataObject(dataObjectWithNoItem);
        assertNotNull(pane.item);
        assertFalse(pane.item.isAvailable());
      }
    
    @Test
    public void setDataObjectWithNoPanel() 
      throws Exception
      {
        panelProvider.setDataObject(dataObjectWithItem);
        assertNull(panelProvider.pane);
      }
  }