/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.viewer;

import java.io.IOException;
import javax.swing.JFrame;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.util.Lookup;
import org.openide.windows.TopComponent;
import it.tidalwave.metadata.spi.TestLoggingSupport;
import it.tidalwave.metadata.viewer.impl.BasicMetadataPanelProvider;
import it.tidalwave.metadata.viewer.mock.MetadataPanelProviderMock;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.Matchers.*;

/*******************************************************************************
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class MetadataViewerTopComponentTest extends TestLoggingSupport
  {
    private MetadataViewerTopComponent viewer;
    private JFrame frame;
    private DataObject dataObject1;
    private DataObject dataObject2;
    
    @Before
    public void setUp() 
      throws IOException 
      {
        viewer = new MetadataViewerTopComponent(); // always create a new one for test isolation
        frame = new JFrame("test");
        frame.getContentPane().add(viewer);
        frame.pack();
        dataObject1 = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("dataobject1"));
        dataObject2 = DataObject.find(FileUtil.createMemoryFileSystem().getRoot().createData("dataobject2"));
        assertNotNull(dataObject1);
        assertNotNull(dataObject2);
        
        // Must reset panelProviders: they are the same instance across al tests, since they are
        // only initialized once by the global Lookup.
        for (final MetadataPanelProvider panelProvider : Lookup.getDefault().lookupAll(MetadataPanelProvider.class))
          {
            panelProvider.setDataObject(null);
            assertNull(panelProvider.getDataObject());
          }
      }  

    @After
    public void tearDown()   
      {
        viewer = null;
        
        if (frame != null)
          {
            frame.dispose(); 
          }
        
        frame = null;
        dataObject1 = null;
        dataObject2 = null;
      }
    
    @Test
    public void testGetInstance()
      {
        final TopComponent c1 = MetadataViewerTopComponent.getDefault();
        final TopComponent c2 = MetadataViewerTopComponent.getDefault();
        assertNotNull(c1);
        assertTrue(c1 instanceof MetadataViewerTopComponent);
        assertTrue(c1 == c2);
      }

    @Test
    public void testFindInstance()
      {
        final TopComponent c1 = MetadataViewerTopComponent.findInstance();
        assertNotNull(c1);
        assertTrue(c1 instanceof MetadataViewerTopComponent);
      }
    
    @Test
    public void testInitialization()
      {
        assertThat(viewer.panelProviders.size(), equalTo(2));
        assertThat(viewer.panelProviders.get(1), instanceOf(BasicMetadataPanelProvider.class));
        assertThat(viewer.panelProviders.get(0), instanceOf(MetadataPanelProviderMock.class));
        assertNull(viewer.pendingDataObject);
        assertFalse(viewer.needToApplyDataObject);
        assertNull(viewer.dataObject);
        assertNull(viewer.pendingDataObject);
        // TODO: assert panel has been created
        assertPanelProvidersSelection(null);
      }
    
    @Test
    public void testPersistence() 
      {
        assertEquals(TopComponent.PERSISTENCE_ALWAYS, viewer.getPersistenceType());
      }
    
    @Test
    public void testPreferredID()
      {
        assertEquals("MetadataViewerTopComponent", viewer.preferredID());    
      }
    
    @Test 
    public void testDataObjectChange()
      {
        frame.setVisible(true);
        viewer.componentShowing();
        assertTrue(viewer.isShowing());
        assertNull(viewer.dataObject);
        assertNull(viewer.pendingDataObject);
        assertFalse(viewer.needToApplyDataObject);
        
        viewer.setDataObject(dataObject1);
        assertTrue(viewer.dataObject == dataObject1);
        assertNull(viewer.pendingDataObject);
        assertFalse(viewer.needToApplyDataObject);
        assertPanelProvidersSelection(dataObject1);
        
        viewer.setDataObject(dataObject2);
        assertTrue(viewer.dataObject == dataObject2);
        assertNull(viewer.pendingDataObject);
        assertFalse(viewer.needToApplyDataObject);
        assertPanelProvidersSelection(dataObject2);
      }
    
    @Test
    public void testComponentDoesntUpdateWhenNotShowing() 
      throws IOException
      {
        assertFalse(viewer.isShowing());
        assertNull("" + viewer.dataObject, viewer.dataObject);
        assertNull(viewer.pendingDataObject);
        assertFalse(viewer.needToApplyDataObject);
        
        // Should not update anything now, since is not showing
        viewer.setDataObject(dataObject1);
        assertNull(viewer.dataObject);
        assertTrue(viewer.pendingDataObject == dataObject1);
        assertTrue(viewer.needToApplyDataObject);
        assertPanelProvidersSelection(null);
        
        // Should not update anything again, since is not showing
        viewer.setDataObject(dataObject2);
        assertNull(viewer.dataObject);
        assertTrue(viewer.pendingDataObject == dataObject2);
        assertTrue(viewer.needToApplyDataObject);
        assertPanelProvidersSelection(null);
        
        // Now the latest dataObject should be applied
        frame.setVisible(true);
        assertTrue(viewer.isShowing());
        viewer.componentShowing();
        assertTrue(viewer.dataObject == dataObject2);
        assertNull(viewer.pendingDataObject);
        assertFalse(viewer.needToApplyDataObject);
        assertPanelProvidersSelection(dataObject2);
      }
    
    private void assertPanelProvidersSelection (final DataObject dataObject)
      {
        for (final MetadataPanelProvider panelProvider : viewer.panelProviders)
          {
            assertTrue(panelProvider.getDataObject() == dataObject);
          }
      }
  }