/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.viewer.impl;

import it.tidalwave.util.logging.Logger;
import java.beans.Introspector;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Collection;
import java.util.Date;
import org.jdesktop.beansbinding.BeanProperty;
import org.jdesktop.beansbinding.Property;
import it.tidalwave.swing.beansbinding.AbstractBean;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class BeanTestSupport<T extends AbstractBean> 
  {
    private static final String CLASS = BeanTestSupport.class.getName();
    private Logger logger = Logger.getLogger(CLASS);
    
    class PropertyChangeListenerTester implements PropertyChangeListener
      {
        public final List<String> latestPropertyName = new ArrayList<String>();
        public final List<Object> latestPropertyValue = new ArrayList<Object>();
        
        public void reset()
          {
            latestPropertyName.clear();
            latestPropertyValue.clear();
          }
        
        public void propertyChange (final PropertyChangeEvent event) 
          {
            if (!event.getPropertyName().equals("available")
                && !event.getPropertyName().equals("metadataLatestModificationTime")) // TODO: ignore it for now, but add in the test later
              {
                latestPropertyName.add(event.getPropertyName());
                latestPropertyValue.add(event.getNewValue());
              }
          }
      }
            
    protected void testAllProperties (final Class<T> beanClass) 
      throws Exception
      {
        for (final PropertyDescriptor propertyDescriptor : Introspector.getBeanInfo(beanClass).getPropertyDescriptors())
          {
            testProperty(beanClass.newInstance(), propertyDescriptor);
          }
      }
    
    private void testProperty (final T bean, final PropertyDescriptor propertyDescriptor)
      {
        String propertyName =  propertyDescriptor.getName();
        final Property<T, Object> property = BeanProperty.<T, Object>create(propertyName);
        final PropertyChangeListenerTester pclt = new PropertyChangeListenerTester();
        bean.addPropertyChangeListener(pclt);
        
        if (propertyDescriptor.getPropertyType().equals(String.class))
          {
            logger.info("Testing class " + bean.getClass() + " property: " + propertyName);
            
            for (int i = 0; i < 10; i++)
              {
                pclt.reset();
                final Object value = "Value " + i;  
                property.setValue(bean, value);
                assertEquals(value, property.getValue(bean));
                
                propertyName = propertyName.replaceAll("IPTC", "iptc");
                
                if (i == 0)
                  {
                    assertEquals(Arrays.asList(propertyName, propertyName + "Available"), pclt.latestPropertyName);
                    assertEquals(Arrays.asList(value, true), pclt.latestPropertyValue);
                  }
                else
                  {
                    assertEquals(Arrays.asList(propertyName), pclt.latestPropertyName);
                    assertEquals(Arrays.asList(value), pclt.latestPropertyValue);
                  }
              }
            
            pclt.reset();
            property.setValue(bean, null);
            assertNull(property.getValue(bean));
          }
        
        else if (propertyDescriptor.getPropertyType().equals(Date.class))
          {
            logger.info("Testing class " + bean.getClass() + " property: " + propertyName);
            
            for (int i = 0; i < 10; i++)
              {
                pclt.reset();
                final Date value = new Date(i * 1000L * 100000 * 300);
                property.setValue(bean, value);
                assertEquals(value, property.getValue(bean));
                
                propertyName = propertyName.replaceAll("IPTC", "iptc");
                
                if (i == 0)
                  {
                    assertEquals(Arrays.asList(propertyName, propertyName + "Available"), pclt.latestPropertyName);
                    assertEquals(Arrays.asList(value, true), pclt.latestPropertyValue);
                  }
                else
                  {
                    assertEquals(Arrays.asList(propertyName), pclt.latestPropertyName);
                    assertEquals(Arrays.asList(value), pclt.latestPropertyValue);
                  }
              }
            
            pclt.reset();
            property.setValue(bean, null);
            assertNull(property.getValue(bean));
          }
        
        else if (Collection.class.isAssignableFrom(propertyDescriptor.getPropertyType()))
          {
            pclt.reset();
            property.setValue(bean, null);
            assertNull(property.getValue(bean));
            
            final Collection<String> value2 = Arrays.asList("1", "2");
            pclt.reset();
            property.setValue(bean, value2);
            
            // Test that you can change the collection contents without affecting the bean
            Collection<Object> collection = (Collection<Object>)property.getValue(bean);
            assertEquals(value2, collection);
            collection.add("3");
            assertEquals(value2.size() + 1, collection.size());
            
            collection = (Collection<Object>)property.getValue(bean);         
            assertEquals(value2, collection);
          }
        
        else if (propertyDescriptor.getPropertyType().equals(boolean.class) && !propertyName.equals("available"))
          {
            logger.info("Testing class " + bean.getClass() + " property: " + propertyName);
            
            for (boolean value = true; value != false; value = false)
              {
                pclt.reset();
                property.setValue(bean, value);
                assertEquals(value, property.getValue(bean));
                
                propertyName = propertyName.replaceAll("IPTC", "iptc");
                
//                if (value == false)
//                  {
//                    assertEquals(Arrays.asList(propertyName, propertyName.replaceAll("Available", "")), pclt.latestPropertyName);
//                    assertEquals(Arrays.asList(false, null), pclt.latestPropertyValue);
//                  }
//                else
                  {
                    assertEquals(Arrays.asList(propertyName), pclt.latestPropertyName);
                    assertEquals(Arrays.asList(value), pclt.latestPropertyValue);
                  }
              }
          }
      }
 }